package org.nakedobjects.applib;

import org.nakedobjects.applib.annotation.Hidden;


/**
 * Convenience super class for all domain objects that wish to interact with the container.
 * 
 * <p>
 * Subclassing is NOT mandatory; the methods in this superclass can be pushed down into domain objects and
 * another superclass used if required.
 * 
 * @see org.nakedobjects.applib.DomainObjectContainer
 */
public abstract class AbstractDomainObject extends AbstractContainedObject {

    protected <T> T newTransientInstance(Class<T> ofClass) {
        return getContainer().newTransientInstance(ofClass);
    }

    /**
     * Resolve this object, populating references to other objects.
     */
    protected void resolve() {
        getContainer().resolve(this);
    }

    /**
     * Resolve this object if the referenced object is still unknown.
     */
    protected void resolve(final Object referencedObject) {
        getContainer().resolve(this, referencedObject);
    }

    /**
     * Notifies the container that this object has changed, so that it can be persisted.
     */
    protected void objectChanged() {
        getContainer().objectChanged(this);
    }

    /**
     * Whether this object is persistent.
     */
    protected boolean isPersistent() {
        return getContainer().isPersistent(this);
    }

    /**
     * Save this object to the persistent object store.
     */
    @Hidden
    protected void makePersistent() {
        getContainer().makePersistent(this);
    }

    @Hidden
    protected void makePersistentIfNotAlready() {
        if (!getContainer().isPersistent(this)) {
            getContainer().makePersistent(this);
        }
    }

    /**
     * Delete this object from the persistent object store.
     */
    protected void disposeInstance() {
        getContainer().disposeInstance(this);
    }
    // }}
}

// Copyright (c) Naked Objects Group Ltd.
