package org.nakedobjects.applib.fixtures;

import java.util.Calendar;
import java.util.TimeZone;

import org.nakedobjects.applib.clock.Clock;


/**
 * This clock, for use by fixtures, can be set to specific time.
 * 
 * <p>
 * If not set it will provide the time provided by the system clock.
 * 
 * <p>
 * Note that - by design - it does not provide any mechanism to advance the time (eg automatic ticking of the
 * clock). That is, the time returned is always explicitly under the control of the programmer (it can be
 * moved forward or back as required).
 */
public class FixtureClock extends Clock {
    private static final TimeZone UTC_TIME_ZONE;
    private Calendar time = null;

    static {
        TimeZone tempTimeZone = TimeZone.getTimeZone("Etc/UTC");
        if (tempTimeZone == null) {
            tempTimeZone = TimeZone.getTimeZone("UTC");
        }
        UTC_TIME_ZONE = tempTimeZone;
    }

    /**
     * Must call before any other call to {@link Clock#getInstance()}.
     * <p>
     * Effectively configures the system to use a FixtureClock rather than the in-built system clock.
     */
    public static FixtureClock initialize() {
        return new FixtureClock();
    }

    private FixtureClock() {}

    /**
     * Access via {@link Clock#getTime()}.
     */
    protected long time() {
        if (time == null) {
            return System.currentTimeMillis();
        } else {
            return time.getTime().getTime();
        }
    }

    public void setTime(final int hour, final int min) {
        getCalendar();
        time.set(Calendar.HOUR_OF_DAY, hour);
        time.set(Calendar.MINUTE, min);
    }

    public void setDate(final int year, final int month, final int day) {
        getCalendar();
        time.set(Calendar.YEAR, year);
        time.set(Calendar.MONTH, month - 1);
        time.set(Calendar.DAY_OF_MONTH, day);
    }

    public void addTime(int hours, int minutes) {
        getCalendar();
        time.add(Calendar.HOUR_OF_DAY, hours);
        time.add(Calendar.MINUTE, minutes);
    }

    public void addDate(int years, int months, int days) {
        getCalendar();
        time.add(Calendar.YEAR, years);
        time.add(Calendar.MONTH, months);
        time.add(Calendar.DAY_OF_MONTH, days);
    }

    private void getCalendar() {
        if (time == null) {
            time = Calendar.getInstance();
            time.setTimeZone(UTC_TIME_ZONE);
        }
    }

    public void reset() {
        time = null;
    }

    public String toString() {
        return time.getTime().toString();
    }
}
// Copyright (c) Naked Objects Group Ltd.
